@@ -7,7 +7,14 @@
 #ifndef _FCITX_CANDIDATELIST_H_
 #define _FCITX_CANDIDATELIST_H_
 
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 #include <fcitx-utils/key.h>
+#include <fcitx-utils/macros.h>
+#include <fcitx/candidateaction.h>
+#include <fcitx/fcitxcore_export.h>
 #include <fcitx/text.h>
 
 namespace fcitx {
@@ -19,6 +26,7 @@ class ModifiableCandidateList;
 class CursorMovableCandidateList;
 class CursorModifiableCandidateList;
 class BulkCursorCandidateList;
+class ActionableCandidateList;
 
 class CandidateListPrivate;
 
@@ -96,6 +104,7 @@ public:
     CursorMovableCandidateList *toCursorMovable() const;
     CursorModifiableCandidateList *toCursorModifiable() const;
     BulkCursorCandidateList *toBulkCursor() const;
+    ActionableCandidateList *toActionable() const;
 
 protected:
     void setPageable(PageableCandidateList *list);
@@ -104,6 +113,7 @@ protected:
     void setCursorMovable(CursorMovableCandidateList *list);
     void setCursorModifiable(CursorModifiableCandidateList *list);
     void setBulkCursor(BulkCursorCandidateList *list);
+    void setActionable(ActionableCandidateList *list);
 
 private:
     std::unique_ptr<CandidateListPrivate> d_ptr;
@@ -124,7 +134,7 @@ public:
     // Following are optional.
     virtual int totalPages() const { return -1; }
     virtual int currentPage() const { return -1; }
-    virtual void setPage(int) {}
+    virtual void setPage(int /*unused*/) {}
 };
 
 class FCITXCORE_EXPORT CursorMovableCandidateList {
@@ -178,7 +188,7 @@ class FCITXCORE_EXPORT DisplayOnlyCandidateWord : public CandidateWord {
 public:
     DisplayOnlyCandidateWord(Text text) : CandidateWord(std::move(text)) {}
 
-    void select(InputContext *) const override {}
+    void select(InputContext * /*inputContext*/) const override {}
 };
 
 class FCITXCORE_EXPORT BulkCursorCandidateList {
@@ -187,6 +197,35 @@ public:
     virtual void setGlobalCursorIndex(int index) = 0;
 };
 
+/**
+ * Interface for trigger actions on candidates.
+ *
+ * @since 5.1.10
+ */
+class FCITXCORE_EXPORT ActionableCandidateList {
+public:
+    virtual ~ActionableCandidateList();
+
+    /**
+     * Check whether this candidate has action.
+     *
+     * This function should be fast and guarantee that candidateActions return a
+     * not empty vector.
+     */
+    virtual bool hasAction(const CandidateWord &candidate) const = 0;
+
+    /**
+     * Return a list of actions.
+     */
+    virtual std::vector<CandidateAction>
+    candidateActions(const CandidateWord &candidate) const = 0;
+
+    /**
+     * Trigger the action based on the index returned from candidateActions.
+     */
+    virtual void triggerAction(const CandidateWord &candidate, int id) = 0;
+};
+
 class DisplayOnlyCandidateListPrivate;
 
 class FCITXCORE_EXPORT DisplayOnlyCandidateList : public CandidateList {
@@ -308,6 +347,13 @@ public:
     void setCursorKeepInSamePage(bool);
     void setCursorPositionAfterPaging(CursorPositionAfterPaging afterPaging);
 
+    /**
+     * Set an optional implemenation of actionable candidate list
+     *
+     * @since 5.1.10
+     */
+    void setActionableImpl(std::unique_ptr<ActionableCandidateList> actionable);
+
 private:
     void fixAfterUpdate();
     void moveCursor(bool prev);
